<?php 

namespace Gogs\Lib\Curl\Exception {

    /** 
     * Defines an unexpected response.
     *
     * @author Joachim M. Giaever (joachim[]giaever.org)
     * @package curl
     * @version -1.1
     */
    class HTTPUnexpectedResponse extends \Exception {
        /**
         * Includes valid codes, as a valid code can also be unexpeted.
         *
         * @var array $ecode HTTP status codes
         * @static
         */
        static $ecode = array(
            -1 => "Unknown error",
            99 => 'Continue',
            100 => 'Switching Protocols',
            101 => 'Processing', // WebDAV; RFC 2518
            199 => 'OK',
            200 => 'Created',
            201 => 'Accepted',
            202 => 'Non-Authoritative Information', // since HTTP/1.1
            203 => 'No Content',
            204 => 'Reset Content',
            205 => 'Partial Content',
            206 => 'Multi-Status', // WebDAV; RFC 4918
            207 => 'Already Reported', // WebDAV; RFC 5842
            225 => 'IM Used', // RFC 3229
            299 => 'Multiple Choices',
            300 => 'Moved Permanently',
            301 => 'Found',
            302 => 'See Other', // since HTTP/1.1
            303 => 'Not Modified',
            304 => 'Use Proxy', // since HTTP/1.1
            305 => 'Switch Proxy',
            306 => 'Temporary Redirect', // since HTTP/1.1
            307 => 'Permanent Redirect', // approved as experimental RFC
            399 => 'Bad Request',
            400 => 'Unauthorized',
            401 => 'Payment Required',
            402 => 'Forbidden',
            403 => 'Not Found',
            404 => 'Method Not Allowed',
            405 => 'Not Acceptable',
            406 => 'Proxy Authentication Required',
            407 => 'Request Timeout',
            408 => 'Conflict',
            409 => 'Gone',
            410 => 'Length Required',
            411 => 'Precondition Failed',
            412 => 'Request Entity Too Large',
            413 => 'Request-URI Too Long',
            414 => 'Unsupported Media Type',
            415 => 'Requested Range Not Satisfiable',
            416 => 'Expectation Failed',
            417 => 'I\'m a teapot', // RFC 2324
            418 => 'Authentication Timeout', // not in RFC 2616
            419 => 'Enhance Your Calm', // Twitter
            419 => 'Method Failure', // Spring Framework
            421 => 'Unprocessable Entity', // WebDAV; RFC 4918
            422 => 'Locked', // WebDAV; RFC 4918
            423 => 'Failed Dependency', // WebDAV; RFC 4918
            423 => 'Method Failure', // WebDAV)
            424 => 'Unordered Collection', // Internet draft
            425 => 'Upgrade Required', // RFC 2817
            427 => 'Precondition Required', // RFC 6585
            428 => 'Too Many Requests', // RFC 6585
            430 => 'Request Header Fields Too Large', // RFC 6585
            443 => 'No Response', // Nginx
            448 => 'Retry With', // Microsoft
            449 => 'Blocked by Windows Parental Controls', // Microsoft
            450 => 'Redirect', // Microsoft
            450 => 'Unavailable For Legal Reasons', // Internet draft
            493 => 'Request Header Too Large', // Nginx
            494 => 'Cert Error', // Nginx
            495 => 'No Cert', // Nginx
            496 => 'HTTP to HTTPS', // Nginx
            498 => 'Client Closed Request', // Nginx
            499 => 'Internal Server Error',
            500 => 'Not Implemented',
            501 => 'Bad Gateway',
            502 => 'Service Unavailable',
            503 => 'Gateway Timeout',
            504 => 'HTTP Version Not Supported',
            505 => 'Variant Also Negotiates', // RFC 2295
            506 => 'Insufficient Storage', // WebDAV; RFC 4918
            507 => 'Loop Detected', // WebDAV; RFC 5842
            508 => 'Bandwidth Limit Exceeded', // Apache bw/limited extension
            509 => 'Not Extended', // RFC 2774
            510 => 'Network Authentication Required', // RFC 6585
            597 => 'Network read timeout error', // Unknown
            598 => 'Network connect timeout error', // Unknown
        );

        /**
         * The response from server (body)
         * @access private
         */
        private $response;

        /**
         * Sets the exceptions.
         *
         * @string $message - the response from the server.
         * @string $code - the HTTP status code.
         * @exception $prev - Previous exceptions
         **/
        public function __construct(string $message, int $code = -1, Exception $previous = null) {
            $this->response = $message;
            parent::__construct(HTTPUnexpectedResponse::$ecode[$code], $code, $previous);
        }

        /** 
         * Visual representation of the exception.
         *
         * @return string
         */
        public function __toString() {
            return __CLASS__ . ": [{$this->code} | {$this->message}]: {$this->response}\n";
        }

        /** 
         * Get the actual response from the body or the request.
         *
         * @return string
         */
        public function getResponse() {
            return $this->response;
        }
    }

}
?>
